<?php

/**
 * @copyright   Copyright (C) 2009 - 2017 Michael RIchey. All rights reserved.
 * @license             GNU General Public License version 2 or later; see LICENSE.txt
 */
// No direct access
defined('_JEXEC') or die;

class plgSystemHeadTag extends JPlugin {

    var $_cache = null;
    var $_doc = null;
    private $_afterrender = null;
    private $_localonly = null;
    private $_host = null;
    private $_user = null;
    private $_menu = null;
    private $_usergroup = null;
    private $_accesslevel = null;
    private $_tagparser = null;
    private $_tagarray = array('scriptdeclaration', 'script', 'styledeclaration', 'stylesheet', 'customtag');
    protected $app;
    protected $_jhtml = array();

    public function __construct($subject, $config) {
        parent::__construct($subject, $config);
        $this->_afterrender = $this->params->get('afterrender', 0);
        if ($this->_afterrender) {
            require_once(__DIR__ . '/tagparser.php');
        }
        $this->_localonly = $this->params->get('localonly', false);
        $this->_user = JFactory::getUser();
        $this->_menu = $this->app->getMenu();
        if ($this->_localonly) {
            $this->_host = JURI::getInstance()->getHost();
        }
        $this->_usergroup = (array) $this->_user->getAuthorisedGroups();
        $this->_accesslevel = (array) $this->_user->getAuthorisedViewLevels();
    }

//	function onBeforeRender() {
    function onBeforeCompileHead() {
        $this->_doc = JFactory::getDocument();
        if ($this->app->isAdmin() || $this->_doc->getType() != 'html') {
            return true;
        }
        $this->_ruleMenuItem();
        $this->_ruleUserGroup();
        $this->_ruleAccessLevel();
    }

    function onContentPrepare($context, &$article, &$params, $page = 0) {
        $this->_doc = JFactory::getDocument();
        if ($this->app->isAdmin() || !$this->_afterrender || $this->_doc->getType() != 'html') {
            return true;
        }
        $this->_setupTagParser();
        $article->text = $this->_scanContent($article->text);
    }

    function onAfterRender() {
        $this->_doc = JFactory::getDocument();
        if ($this->app->isAdmin() || !$this->_afterrender || $this->_doc->getType() != 'html') {
            return true;
        }
        $this->_setupTagParser();
        $body = $this->_scanContent(JResponse::getBody());
        JResponse::setBody($body);
    }

    private function _ruleMenuItem() {
        $activemenu = $this->_menu->getActive();
        if(!is_object($activemenu)) {
            return;
        }
        $parents = ($activemenu->level > 1) ? $this->_getParents($activemenu) : array($activemenu->id);
        $itemid = $activemenu->id;
        foreach ((array) $this->params->get('menuitem') as $rule) {
            if (property_exists($rule,'menuitem') && is_array($rule->menuitem) &&
                    (property_exists($rule, 'children') && (int) $rule->children == 1 && array_intersect($parents, $rule->menuitem)) ||
                    (in_array($itemid, $rule->menuitem))
            ) {
                $this->_headtag($rule);
            }
        }
    }

    private function _ruleUserGroup() {
        foreach ((array) $this->params->get('usergroup') as $rule) {
            if (array_intersect($this->_user->groups, $rule->usergroup)) {
                $this->_headtag($rule);
            }
        }
    }

    private function _ruleAccessLevel() {
        foreach ((array) $this->params->get('accesslevel') as $rule) {
            if (array_intersect($this->_accesslevel, $rule->accesslevel)) {
                $this->_headtag($rule);
            }
        }
    }

    private function _getParents($item) {
        $parents = array($item->id);
        while ($item->level > 1) {
            $item = $this->_menu->getItem($item->parent_id);
            $parents[] = $item->id;
        }
        return $parents;
    }

    private function _setupTagParser() {
        if (is_null($this->_tagparser)) {
            $this->_tagparser = plg_system_headtag\TagParser::getInstance('headtag', array('alternateclose' => 'headtag'));
        }
    }

    private function _scanContent($body) {
        if (strpos($body, '{headtag:') === false) {
            return $body; // only operate on pages that contain the headtag code
        }
        $cmd = $this->app->input->getCmd('option');
        foreach ($this->_tagarray as $type) {
            $this->_tagparser->setIdentifier('headtag:' . $type)->resetTags()->parse($body);
            foreach ($this->_tagparser->tags as $tag) {
                $body = $this->_scanContentTag($tag,$type,$cmd,$body);
            }
        }
        return $body;
    }
    
    private function _scanContentTag($tag,$type,$cmd,$body) {
        $accesslevels = $tag->getParam('accesslevel', false);
        $usergroups = $tag->getParam('usergroup', false);
        $jhtml = $tag->getParam('loadframework','');
        if (
                ($accesslevels && array_intersect($this->_accesslevel, explode(',', $accesslevels))) || 
                ($usergroups && array_intersect($this->_usergroup, explode(',', $usergroups))) ||
                (!$accesslevels && !$usergroups)
        ) {
            $rule = $this->_buildRule($type, $tag);
            if (!($this->params->get('omitsearch', 1) && in_array($cmd, array('com_finder', 'com_search')))) { // scrub
                $this->_loadJHtml($jhtml);
                $this->_headtag($rule);
            }
        }
        if (!is_null($tag)) {
            $body = $tag->replace($body, '');
        }
        return $body;
    }

    private function _buildRule($type, $tag) {
        $rule = new stdClass();
        $rule->type = $type;
        $rule->script_stylesheet = $tag->content;
        $rule->declaration_custom = $tag->content;
        $rule->attribs = array();
        $rule->options = array();
        if ($tag->getParam('defer', false)) {
            $rule->attribs['defer'] = 'defer';
        }
        if ($tag->getParam('async', false)) {
            $rule->attribs['async'] = 'async';
        }
        if ($tag->getParam('mediaversion',false)) {
            $rule->options['version'] = 'auto';
        }
        if ($conditional = $tag->getParam('conditional',false) || property_exists($rule,'conditional')?$rule->conditional:false) {
            $rule->options['conditional'] = $conditional?:$rule->conditional;
        }
        return $rule;
    }

    private function _headtag($rule) {
        if (property_exists($rule, 'disabled') && $rule->disabled) {
            return;
        }
        $types = array('scriptdeclaration' => 'addScriptDeclaration', 'styledeclaration' => 'addStyleDeclaration', 'customtag' => 'addCustomTag');
        if(property_exists($rule,'mediaversion') && $rule->mediaversion) { $rule->options['version'] = 'auto'; }
        $this->_loadJHtml(isset($rule->loadframework)?$rule->loadframework:false);
        switch ($rule->type) {
            case 'script':
                if($rule->defer) { $rule->attribs['defer'] = 'defer'; }
                if($rule->async) { $rule->attribs['async'] = 'async'; }
                if($rule->conditional) { $rule->options['conditional'] = $rule->conditional; }
                $this->_addScript($rule->script_stylesheet, $rule->options, $rule->attribs);
                break;
            case 'stylesheet':
                $this->_addStyleSheet($rule->script_stylesheet, (isset($rule->options)?$rule->options:array()), (isset($rule->attribs)?$rule->attribs:array()));
                break;
            case 'scriptdeclaration':
            case 'styledeclaration':
            case 'customtag':
                $this->_doc->{$types[$rule->type]}($rule->declaration_custom);
                break;
        }
    }
    
    private function _loadJHtml($jhtml) {
        if(!$jhtml) {
            return;
        }
        foreach(array_filter(explode(',',$jhtml)) as $j) {
            JHtml::_($j);
        }
    }

    private function _addScript($link, $options, $attribs) {
        if ($this->_localonly && !$this->_verifyLocal($link)) {
            return;
        }
        $this->_doc->addScript($link, $options, $attribs);
    }

    private function _addStyleSheet($link, $options, $attribs) {
        if ($this->_localonly && !$this->_verifyLocal($link)) {
            return;
        }
        $this->_doc->addStyleSheet($link, $options, $attribs);
    }

    private function _verifyLocal($link) {
        $url = parse_url($link);
        if (isset($url['host']) && $url['host'] != $this->_host) {
            return false;
        }
        return true;
    }

}
